@@ -88,6 +88,8 @@ logical function isanalysis(string,noprint)
     isanalysis = .true.
  case('lightcurve')
     isanalysis = .true.
+ case('extinction')
+    isanalysis = .true.
  case('none')
     verbose = .false.
  end select
@@ -104,6 +106,8 @@ logical function isanalysis(string,noprint)
     print "(a)",'                             output to file called ''energy.out'''
     print "(a)",'         calc massaboverho : mass above a series of density thresholds vs time'
     print "(a)",'                             output to file called ''massaboverho.out'''
+    print "(a)",'         calc extinction   : column density to all sink particles vs time'
+    print "(a)",'                             output to file called ''extinction.out'''
 !    print "(a)",'         calc rhomach      : density variance and RMS velocity dispersion vs. time'
 !    print "(a)",'                             output to file called ''rhomach.out'''
     print "(a)",'         calc max          : maximum of each column vs. time'
@@ -130,6 +134,8 @@ logical function isanalysis(string,noprint)
     print "(a)",'                             output to file called ''time_average.out'''
     print "(/,a)",'         calc ratio        : ratio of *all* entries in each file compared to first'
     print "(a)",'                             output to file called ''ratio.out'''
+    print "(/,a)",'         calc plus        : add two snapshots together'
+    print "(a)",'                             output to file called ''plus.out'''
  elseif (.not.isanalysis .and. doprint) then
     print "(a)",'Analysis mode:'
     print "(a)",'   splash calc <mode>  : type "splash calc" for details'
@@ -142,18 +148,21 @@ end function isanalysis
 !  open output file/ initialise quantities needed for analysis
 !  over all dump files
 !----------------------------------------------------------------
-subroutine open_analysis(analysistype,required,ncolumns,ndim,ndimV)
- use labels,       only:ix,ivx,ih,iBfirst,iutherm,irho,ipmass,itemp,ikappa,label
- use asciiutils,   only:read_asciifile,basename
- use filenames,    only:rootname,nfiles,tagline,fileprefix,ifileopen
- use params,       only:maxplot
- use system_utils, only:ienvlist
- integer, intent(in) :: ncolumns,ndim,ndimV
+subroutine open_analysis(analysistype,required,ncolumns,ndim,ndimV,nsinks)
+ use labels,        only:ix,ivx,ih,iBfirst,iutherm,irho,ipmass,itemp,ikappa,label,&
+                         lenunitslabel,unitslabel,labelzintegration,get_unitlabel_coldens
+ use asciiutils,    only:read_asciifile,basename,integer_to_string
+ use filenames,     only:rootname,nfiles,tagline,fileprefix,ifileopen
+ use params,        only:maxplot
+ use system_utils,  only:ienvlist
+ use settings_data, only:iRescale
+ integer, intent(in) :: ncolumns,ndim,ndimV,nsinks
  character(len=*), intent(in) :: analysistype
  logical, dimension(0:ncolumns), intent(out) :: required
  character(len=maxplot*18) :: headerline   ! len=maxplot x 18 characters
  character(len=64) :: levelsfile
  character(len=maxplot*12) :: fmtstring
+ character(len=lenunitslabel) :: labelt,labelc
  logical :: iexist,standardheader
  integer :: ierr,i,lunit
 !
@@ -168,6 +177,10 @@ subroutine open_analysis(analysistype,required,ncolumns,ndim,ndimV)
  headerline = ' '
  standardheader = .false.
  nfileout = 1
+ labelt = ''
+ if (iRescale) then
+    labelt = unitslabel(0)
+ endif
 
  select case(trim(analysistype))
  case('energy','energies')
@@ -394,9 +407,30 @@ subroutine open_analysis(analysistype,required,ncolumns,ndim,ndimV)
         fileout = 'lightcurve.out'
      endif
      write(headerline,"('#',8(1x,'[',i2.2,1x,a11,']',2x))") &
-           1,'time',2,'Luminosity',3,'R_{eff}',4,'T_{eff}',&
+           1,'time'//trim(labelt),2,'Luminosity',3,'R_{eff}',4,'T_{eff}',&
            5,'L_{bol}',6,'R_{bb}',7,'T_c'
 
+  case('extinction')
+     !
+     !--for extinction of stars need h, mass and density
+     !
+     required(ix(1:ndim)) = .true.
+     required(ih) = .true.
+     required(ipmass) = .true.
+     required(irho) = .true.
+     !
+     !--set filename and header line
+     !
+     if (nfiles==1) then
+        fileout = 'extinction_'//trim(basename(rootname(ifileopen)))//'.out'
+     else
+        fileout = 'extinction.out'
+     endif
+     labelc = get_unitlabel_coldens(iRescale,labelzintegration,unitslabel(irho))
+
+     write(fmtstring,"('(''#'',1x,',i3,'(''['',i2.2,1x,a15,'']'',2x))')",iostat=ierr) nsinks+1
+     write(headerline,fmtstring) &
+           1,'time'//trim(labelt),(i+1,'sink'//trim(adjustl(integer_to_string(i)))//trim(labelc),i=1,nsinks)
  end select
 
  if (standardheader) then
@@ -405,7 +439,7 @@ subroutine open_analysis(analysistype,required,ncolumns,ndim,ndimV)
 !  (this is to avoid repeated code above)
 !
     write(fmtstring,"('(''#'',1x,',i3,'(''['',i2.2,1x,a12,'']'',2x))')",iostat=ierr) ncolumns+1
-    write(headerline,fmtstring) 1,'time',(i+1,label(i)(1:12),i=1,ncolumns)
+    write(headerline,fmtstring) 1,'time'//trim(labelt),(i+1,label(i)(1:12),i=1,ncolumns)
  endif
 
 !
@@ -451,7 +485,6 @@ subroutine open_analysis(analysistype,required,ncolumns,ndim,ndimV)
  !endif
  nfilesread = 0
 
- return
 end subroutine open_analysis
 
 !----------------------------------------------------------------
@@ -461,16 +494,18 @@ end subroutine open_analysis
 !----------------------------------------------------------------
 subroutine write_analysis(time,dat,ntot,ntypes,npartoftype,massoftype,&
                           iamtype,ncolumns,ndim,ndimV,analysistype)
- use labels,        only:ix,ivx,iBfirst,iutherm,irho,ipmass,labeltype,label
+ use labels,        only:ix,ivx,iBfirst,iutherm,irho,ipmass,labeltype,label,&
+                         lenunitslabel,get_unitlabel_coldens,unitslabel,labelzintegration
  use params,        only:int1,doub_prec,maxplot
  use asciiutils,    only:ucase,basename
  use system_utils,  only:renvironment
  use settings_part, only:iplotpartoftype
  use particle_data, only:time_was_read
- use settings_data, only:xorigin,icoords,icoordsnew,itracktype,itrackoffset
+ use settings_data, only:xorigin,icoords,icoordsnew,track_string,iRescale
  use geomutils,     only:change_coords
  use part_utils,    only:get_tracked_particle
  use lightcurve,    only:get_lightcurve
+ use extinction,    only:get_extinction
  use filenames,     only:rootname,ifileopen
  use vectorutils,   only:cross_product3D
  integer, intent(in)               :: ntot,ntypes,ncolumns,ndim,ndimV
@@ -481,7 +516,7 @@ subroutine write_analysis(time,dat,ntot,ntypes,npartoftype,massoftype,&
  real, intent(in), dimension(:,:)  :: dat
  character(len=*), intent(in)      :: analysistype
  real(kind=doub_prec), dimension(maxlevels) :: massaboverho
- integer              :: itype,i,j,ierr,ntot1,ncol1,nused,itrack,ifile
+ integer              :: itype,i,j,ierr,ntot1,ncol1,nused,itrack,ifile,nsinks
  real(kind=doub_prec) :: ekin,emag,etherm,epot,etot,totmom,pmassi,totang
  real(kind=doub_prec) :: totvol,voli,rhoi,rmsvmw,v2i
  real(kind=doub_prec) :: rhomeanmw,rhomeanvw,rhovarmw,rhovarvw,bval,bvalmw
@@ -493,16 +528,22 @@ subroutine write_analysis(time,dat,ntot,ntypes,npartoftype,massoftype,&
  character(len=20)    :: fmtstring
  logical              :: change_coordsys
  real                 :: x0(3),v0(3)
+ character(len=lenunitslabel) :: labelt,labelc
 !
 ! array with one value for each column
 !
  real(kind=doub_prec) :: coltemp(maxplot), vals(maxplot), rmsval(maxplot)
+ real :: coltemps(maxplot)
+
+ labelt = ''
+ labelc = ''
+ if (iRescale) labelt = unitslabel(0)
 
  nfilesread = nfilesread + 1
  if (time_was_read(time)) then
     timei = time
-    print "(/,5('-'),a,', TIME=',es9.2,' FILE #',i5,/)",&
-          '> CALCULATING '//trim(ucase(analysistype)),time,nfilesread
+    print "(/,5('-'),a,', TIME=',es9.2,a,' FILE #',i5,/)",&
+          '> CALCULATING '//trim(ucase(analysistype)),time,trim(labelt),nfilesread
  else
     timei = 0.
     print "(/,5('-'),a,', FILE #',i5,' (TIME NOT READ)'/)",&
@@ -512,11 +553,12 @@ subroutine write_analysis(time,dat,ntot,ntypes,npartoftype,massoftype,&
  change_coordsys = (icoordsnew /= icoords .and. ndim > 0 .and. all(ix(1:ndim) > 0))
  x0 = xorigin(:)  ! note that it is not currently possible to do splash to ascii
  v0 = 0.          ! with coords set relative to a tracked particle, so just use xorigin
+                  ! instead, one can use the --origin flag to make positions and velocities relative to a particle
 
  if (itracks(1) > 0) then
     itrack = itracks(1)  ! override particle id saved to splash.defaults file if --tracks specified
  else
-    itrack = get_tracked_particle(itracktype,itrackoffset,npartoftype,iamtype)
+    itrack = get_tracked_particle(track_string,npartoftype,iamtype,dat,irho)
  endif
  if (itrack==0) itrack = 1
 
@@ -1180,7 +1222,7 @@ subroutine write_analysis(time,dat,ntot,ntypes,npartoftype,massoftype,&
        endif
     endif
     return
-  case('lightcurve')
+ case('lightcurve')
      call get_lightcurve(ncolumns,dat,npartoftype,massoftype,iamtype,ndim,ntypes,&
          lum,rphoto,tphoto,l_bb,r_bb,t_bb,basename(rootname(ifileopen)))
      print "(4(/,1x,a20,' = ',es9.2))",'Luminosity',lum,'photospheric radius ',rphoto,'photospheric temperature',tphoto
@@ -1189,6 +1231,15 @@ subroutine write_analysis(time,dat,ntot,ntypes,npartoftype,massoftype,&
      !
      write(iunit,"(7(es18.10,1x))") timei,lum,rphoto,tphoto,l_bb,r_bb,t_bb
 
+ case('extinction')
+     call get_extinction(ncolumns,dat,npartoftype,massoftype,iamtype,ndim,ntypes,nsinks,coltemps)
+     labelc = get_unitlabel_coldens(iRescale,labelzintegration,unitslabel(irho))
+     print "(100(/,1x,a20,i0,' = ',es9.2,a))",('Sigma to sink ',i,coltemps(i),trim(labelc),i=1,nsinks)
+     !
+     !--write line to output file
+     !
+     write(iunit,"(100(es18.10,5x))") timei,coltemps(1:nsinks)
+
  case default
     print "(a)",' ERROR: unknown analysis type in write_analysis routine'
     return
