@@ -32,9 +32,9 @@
 !---------------------------------------------------------------------------
 module asciiutils
  implicit none
- public :: read_asciifile,get_ncolumns,get_nrows,ncolumnsline,safename,basename
+ public :: read_asciifile,get_ncolumns,get_nrows,ncolumnsline,safename,basename,numfromfile
  public :: cstring,fstring,add_escape_chars
- public :: string_replace, string_delete, nheaderlines, string_sub
+ public :: string_replace,string_delete,get_nheaderlines,string_sub
  public :: ucase,lcase,strip
  public :: get_line_containing
  public :: enumerate,isdigit,get_digits,integer_to_string,split
@@ -43,7 +43,7 @@ module asciiutils
  public :: match_column,match_tag_start,match_integer,match_lists
  public :: count_non_blank,find_repeated_tags,count_char
  public :: get_extensions,readline_csv,extension
- public :: reorder_filenames_for_comparison
+ public :: sort_filenames_for_comparison
  public :: read_var_from_file
  integer, parameter :: max_line_length = 10000 ! for finding number of columns
 
@@ -399,7 +399,7 @@ subroutine get_ncolumns(lunit,ncolumns,nheaderlines,csv,maxheaderlines)
  integer, intent(out) :: ncolumns,nheaderlines
  integer, intent(in), optional :: maxheaderlines
  logical, intent(in), optional :: csv
- integer :: ierr,ncolprev,ncolsthisline,maxlines,ncolstot
+ integer :: ierr,ncolprev,ncolprev2,ncolsthisline,maxlines,ncolstot
  character(len=max_line_length) :: line
  logical :: nansinfile,infsinfile,is_csv
 
@@ -413,6 +413,7 @@ subroutine get_ncolumns(lunit,ncolumns,nheaderlines,csv,maxheaderlines)
  ierr = 0
  ncolumns = 0
  ncolprev = -100
+ ncolprev2 = -200
  ncolsthisline = 0
  ncolstot = 0
  nansinfile = .false.
@@ -421,9 +422,12 @@ subroutine get_ncolumns(lunit,ncolumns,nheaderlines,csv,maxheaderlines)
  if (present(csv)) is_csv = csv
 !
 !--loop until we find two consecutive lines with the same number of columns (but non zero)
+!  if ncolumns==1 then we must find 3 consecutive lines
 !
- do while ((len_trim(line)==0 .or. ncolsthisline /= ncolprev .or. ncolumns <= 1) &
+ do while ((len_trim(line)==0 .or. ncolsthisline /= ncolprev .or. ncolumns < 1 .or. &
+           (ncolumns==1 .and. ncolsthisline /= ncolprev2)) &
            .and. ierr==0 .and. nheaderlines <= maxlines)
+    ncolprev2 = ncolprev
     ncolprev = ncolumns
     read(lunit,"(a)",iostat=ierr) line
     if (index(line,'NaN') > 0) nansinfile = .true.
@@ -439,6 +443,7 @@ subroutine get_ncolumns(lunit,ncolumns,nheaderlines,csv,maxheaderlines)
  enddo
  !--subtract 2 from the header line count (the last two lines which were the same)
  nheaderlines = max(nheaderlines - 2,0)
+ if (ncolumns==1) nheaderlines = max(nheaderlines - 1,0)
  if (is_csv) ncolumns = ncolstot
 
  if (ierr  > 0 .or. ncolumns <= 0) then
@@ -591,7 +596,7 @@ end subroutine readline_csv
 !
 ! File must be open and at the desired starting position
 !----------------------------------------------------------------------
-integer function nheaderlines(lunit)
+integer function get_nheaderlines(lunit) result(nheaderlines)
  integer, intent(in) :: lunit
  real    :: dum
  integer :: ierr
@@ -605,7 +610,7 @@ integer function nheaderlines(lunit)
  enddo
  nheaderlines = nheaderlines - 1
 
-end function nheaderlines
+end function get_nheaderlines
 
 !---------------------------------------------------------------------------
 !
@@ -765,17 +770,20 @@ subroutine string_replace(string,skey,sreplacewith)
  character(len=*), intent(inout) :: string
  character(len=*), intent(in)    :: skey,sreplacewith
  character(len=len(string)) :: remstring
- integer :: ipos,imax,lensub,i
+ integer :: ipos,imax,lensub,i,iposnext
 
  ipos = index(trim(string),skey)
  lensub = len(skey)
  imax   = len(string)
  i = 0
  do while (ipos > 0 .and. i <= imax)
+    iposnext = ipos + len(sreplacewith)
     i = i + 1  !  only allow as many replacements as characters
     remstring = string(ipos+lensub:len_trim(string))
     string = string(1:ipos-1)//sreplacewith//remstring
-    ipos = index(trim(string),skey)
+    ipos = index(trim(string(iposnext:)),skey)
+    if (ipos > 0) ipos = ipos + iposnext
+    !print*,ipos,' string = ',trim(string),'iposnext = ',trim(string(iposnext:))
  enddo
 
 end subroutine string_replace
@@ -1627,30 +1635,39 @@ integer function numfromfile(filename)
 end function numfromfile
 
 !------------------------------------------------------------
-! utility to reorder a list of files
-! NOT YET IMPLEMENTED...
+! utility to reorder a list of files. We use an insertion
+! sort algorithm which preserves relative order of filenames
+! so a list of files like:
+!
+!  disc_00000 disc_00001 disc_00002 disc1_00000 disc1_00002
+!
+! will be reordered as:
+!
+!  disc_00000 disc1_00000 disc_00001 disc1_00001 disc_00002
+!
 !------------------------------------------------------------
-subroutine reorder_filenames_for_comparison(nfiles,filenames)
+subroutine sort_filenames_for_comparison(nfiles,filenames)
  integer, intent(in) :: nfiles
  character(len=*), intent(inout) :: filenames(nfiles)
- integer :: i,n,nprev,nseq
+ character(len=100) :: key
+ integer :: i,j,num
 
- nseq = 0
- nprev = 0
- do i=1,nfiles
-    n = numfromfile(filenames(i))
-    if (i > 1) then
-       if (n == nprev + 1) then
-          nseq = nseq + 1
-          print*,i,n,nseq,trim(filenames(i))
-       else
-          nseq = 0
-       endif
-       nprev = n
-    endif
+ do i = 2, nfiles
+    key = filenames(i)
+    num = numfromfile(key)
+    j = i - 1
+
+    ! move elements of the filenames array with number
+    ! greater than the key to one position ahead 
+    ! of their current position
+    do while(j >= 1 .and. numfromfile(filenames(j)) > num)
+       filenames(j+1) = filenames(j)
+       j = j - 1
+    enddo
+    filenames(j+1) = key
  enddo
 
-end subroutine reorder_filenames_for_comparison
+end subroutine sort_filenames_for_comparison
 
 !------------------------------------------------------------
 ! utility to read a variable from an ascii file
