@@ -31,8 +31,10 @@ module readwrite_fits
  public :: read_fits_image,write_fits_image,fits_error
  public :: read_fits_cube,write_fits_cube
  public :: read_fits_header
- public :: get_floats_from_fits_header,get_from_header
+ public :: get_floats_from_fits_header,get_from_header,get_from_header_s
  public :: append_to_fits_cube
+ public :: get_velocity_from_fits_header
+ public :: flatten_header
 
  interface write_fits_image
   module procedure write_fits_image,write_fits_image64
@@ -65,7 +67,7 @@ contains
 subroutine read_fits_image(filename,image,naxes,ierr,hdr)
  character(len=*), intent(in)   :: filename
  real(kind=real32), intent(out), allocatable :: image(:,:)
- character(len=:), intent(inout), allocatable, optional :: hdr(:)
+ character(len=80), intent(inout), allocatable, optional :: hdr(:)
  integer, intent(out) :: naxes(2),ierr
  integer :: iunit,ireadwrite,npixels,blocksize
  integer :: firstpix,nullval,group,nfound!,bitpix
@@ -124,7 +126,7 @@ end subroutine read_fits_image
 !---------------------------------------------------
 subroutine read_fits_header(filename,hdr,ierr)
  character, intent(in)  :: filename
- character(len=:), allocatable, intent(out) :: hdr(:)
+ character(len=80), allocatable, intent(out) :: hdr(:)
  integer, intent(out) :: ierr
  integer :: ireadwrite,iunit,blocksize
 
@@ -149,7 +151,7 @@ end subroutine read_fits_header
 subroutine read_fits_head(iunit,hdr,ierr)
  integer, intent(in)  :: iunit
  integer, intent(out) :: ierr
- character(len=:), allocatable, intent(inout) :: hdr(:)
+ character(len=80), allocatable, intent(inout) :: hdr(:)
  character(len=80) :: record
  integer :: i,nkeys,nspace
 
@@ -160,7 +162,7 @@ subroutine read_fits_head(iunit,hdr,ierr)
  ! allocate memory
  !
  if (allocated(hdr)) deallocate(hdr)
- allocate(character(80) :: hdr(nkeys))
+ allocate(hdr(nkeys))
 
 ! Read each 80-character keyword record, and print it out.
  do i = 1, nkeys
@@ -182,16 +184,27 @@ subroutine write_fits_head(iunit,hdr,ierr)
  integer :: i,morekeys
 
  ierr = 0
- morekeys = size(hdr)
+ morekeys = 0
+ ! count non-blank header lines
+ do i=1,size(hdr)
+    if (len_trim(hdr(i)) > 0) then
+       morekeys = morekeys + 1
+    endif
+ enddo
+ ! write header size to fits file
  call fthdef(iunit,morekeys,ierr)
  if (ierr /= 0) return
+
+ ! write header
  do i=1,size(hdr)
-    select case(hdr(i)(1:6))
-    case('SIMPLE','BITPIX','NAXIS ','NAXIS1','NAXIS2','NAXIS3','NAXIS4','EXTEND')
-       ! skip the above keywords
-    case default
-       call ftprec(iunit,hdr(i),ierr)
-    end select
+    if (len_trim(hdr(i)) > 0) then
+       select case(hdr(i)(1:6))
+       case('SIMPLE','BITPIX','NAXIS ','NAXIS1','NAXIS2','NAXIS3','NAXIS4','EXTEND')
+          ! skip the above keywords
+       case default
+          call ftprec(iunit,hdr(i),ierr)
+       end select
+    endif
  enddo
 
 end subroutine write_fits_head
@@ -200,10 +213,11 @@ end subroutine write_fits_head
 ! subroutine to read spectral cube from FITS file
 ! using cfitsio library
 !---------------------------------------------------
-subroutine read_fits_cube(filename,image,naxes,ierr,hdr,hdu)
+subroutine read_fits_cube(filename,image,naxes,ierr,hdr,hdu,velocity)
  character(len=*), intent(in)   :: filename
  real(kind=real32), intent(out), allocatable :: image(:,:,:)
- character(len=:), intent(inout), allocatable, optional :: hdr(:)
+ character(len=80), intent(inout), allocatable, optional :: hdr(:)
+ real(kind=real32), intent(out), allocatable, optional :: velocity(:)
  integer, intent(out) :: naxes(4),ierr
  integer, intent(in), optional :: hdu ! specify which hdu to read
  integer :: iunit,ireadwrite,npixels,blocksize
@@ -241,6 +255,11 @@ subroutine read_fits_cube(filename,image,naxes,ierr,hdr,hdu)
  call ftgisz(iunit,3,naxes(1:ndim),ierr)
  if (ndim==2) naxes(3) = 1
 
+ if (present(hdr) .and. present(velocity) .and. ndim >= 3) then
+    if (.not.allocated(velocity)) allocate(velocity(naxes(3)))
+    call get_velocity_from_fits_header(naxes(3),velocity,hdr,ierr)
+ endif
+
  !if (present(hdr)) bitpix = abs(get_from_header('BITPIX',hdr,ierr))
 
  ! call ftgknj(iunit,'NAXIS',1,2,naxes,nfound,ierr)
@@ -492,7 +511,7 @@ end subroutine write_fits_cube64
 subroutine read_fits_image64(filename,image,naxes,ierr,hdr)
  character(len=*), intent(in)   :: filename
  real(kind=real64), intent(out), allocatable :: image(:,:)
- character(len=:), intent(inout), allocatable, optional :: hdr(:)
+ character(len=80), intent(inout), allocatable, optional :: hdr(:)
  integer, intent(out) :: naxes(2),ierr
  real(kind=real32), allocatable :: img32(:,:)
 
@@ -509,20 +528,27 @@ end subroutine read_fits_image64
 !-------------------------------------------------------------
 ! read fits cube and convert to double precision
 !-------------------------------------------------------------
-subroutine read_fits_cube64(filename,image,naxes,ierr,hdr)
+subroutine read_fits_cube64(filename,image,naxes,ierr,hdr,velocity)
  character(len=*), intent(in)   :: filename
  real(kind=real64), intent(out), allocatable :: image(:,:,:)
- character(len=:), intent(inout), allocatable, optional :: hdr(:)
+ character(len=80), intent(inout), allocatable, optional :: hdr(:)
+ real(kind=real64), intent(inout), allocatable, optional :: velocity(:)
  integer, intent(out) :: naxes(4),ierr
- real(kind=real32), allocatable :: img32(:,:,:)
+ real(kind=real32), allocatable :: img32(:,:,:),velocity32(:)
 
  if (present(hdr)) then
-    call read_fits_cube(filename,img32,naxes,ierr,hdr)
+    if (present(velocity)) then
+       call read_fits_cube(filename,img32,naxes,ierr,hdr=hdr,velocity=velocity32)
+       velocity = velocity32  ! allocate and copy, converting real type
+    else
+       call read_fits_cube(filename,img32,naxes,ierr,hdr)
+    endif
  else
     call read_fits_cube(filename,img32,naxes,ierr)
  endif
  image = img32  ! allocate and copy, converting real type
  deallocate(img32,stat=ierr)
+ if (allocated(velocity32)) deallocate(velocity32,stat=ierr) 
 
 end subroutine read_fits_cube64
 
@@ -551,42 +577,61 @@ end subroutine get_floats_from_fits_header
 ! will extract anything readable as a floating
 ! point number
 !------------------------------------------------
-subroutine get_fits_header_entry(record,tag,val,ierr)
+subroutine get_fits_header_entry(record,key,rval,ierr)
  character(len=80), intent(in) :: record
- character(len=*),  intent(out) :: tag
- real, intent(out) :: val
+ character(len=*),  intent(out) :: key
+ real, intent(out) :: rval
  integer, intent(out) :: ierr
- integer :: ieq
+ character(len=80) :: val
 
- tag = ''
- val = 0.
- ierr = -1
- ! split on equals sign
- ieq = index(record,'=')
- if (ieq > 0) then
-    tag = record(1:ieq-1)
-    read(record(ieq+1:),*,iostat=ierr) val
+ call get_fits_header_key_val(record,key,val,ierr)
+ if (ierr == 0) then
+    read(val,*,iostat=ierr) rval
  endif
 
 end subroutine get_fits_header_entry
 
+!------------------------------------------------
+! get tag:val pairs from fits header record
+! returns the string value
+!------------------------------------------------
+subroutine get_fits_header_key_val(record,key,val,ierr)
+ character(len=80), intent(in) :: record
+ character(len=*),  intent(out) :: key
+ character(len=*),  intent(out) :: val
+ integer, intent(out) :: ierr
+ integer :: ieq
+
+ key = ''
+ val = ''
+ ierr = -1
+ ! split on equals sign
+ ieq = index(record,'=')
+ if (ieq > 0) then
+    key = record(1:ieq-1)
+    val = record(ieq+1:)
+    ierr = 0
+ endif
+
+end subroutine get_fits_header_key_val
+
 !------------------------------------------------
 ! search fits header to find a particular variable
 ! e.g. bmaj = get_from_header('BMAJ',hdr,ierr)
 !------------------------------------------------
-function get_from_header(tag,hdr,ierr) result(val)
- character(len=*),  intent(in) :: tag
+function get_from_header(key,hdr,ierr) result(val)
+ character(len=*),  intent(in) :: key
  character(len=80), intent(in) :: hdr(:)
  integer,           intent(out) :: ierr
- character(len=len(tag)) :: mytag
+ character(len=len(key)) :: mykey
  real :: val,myval
  integer :: i
 
  val = 0.
  ierr = -1
  do i=1,size(hdr)
-    call get_fits_header_entry(hdr(i),mytag,myval,ierr)
-    if (trim(adjustl(mytag))==trim(tag) .and. ierr==0) then
+    call get_fits_header_entry(hdr(i),mykey,myval,ierr)
+    if (trim(adjustl(mykey))==trim(key) .and. ierr==0) then
        val = myval
        ierr = 0
        return
@@ -595,4 +640,101 @@ function get_from_header(tag,hdr,ierr) result(val)
 
 end function get_from_header
 
+!------------------------------------------------
+! search fits header to find a particular string
+! e.g. bmaj = get_from_header('BMAJ',hdr,ierr)
+!------------------------------------------------
+function get_from_header_s(key,hdr,ierr) result(val)
+ character(len=*),  intent(in) :: key
+ character(len=80), intent(in) :: hdr(:)
+ integer,           intent(out) :: ierr
+ character(len=len(key)) :: mykey
+ character(len=80) :: val,myval
+ integer :: i,i1,i2,islash
+
+ val = ''
+ ierr = -1
+ do i=1,size(hdr)
+    call get_fits_header_key_val(hdr(i),mykey,myval,ierr)
+    if (trim(adjustl(mykey))==trim(key) .and. ierr==0) then
+       i1 = index(myval,"'")
+       islash = index(myval,"/")
+       if (islash == 0) islash = len_trim(myval)
+       i2 = index(myval(1:islash),"'",back=.true.)
+       if (i1 > 0) then
+          val = myval(i1+1:i2-1)  ! trim quotation marks
+       else
+          val = myval ! not a string variable, return whole string
+       endif
+       ierr = 0
+       return
+    endif
+ enddo
+
+end function get_from_header_s
+
+!------------------------------------------------
+! delete third dimension in the fits header
+!------------------------------------------------
+subroutine flatten_header(hdr)
+ character(len=80), intent(inout) :: hdr(:)
+ character(len=80) :: mykey,myval
+ integer :: i,ierr
+
+ do i=1,size(hdr)
+    call get_fits_header_key_val(hdr(i),mykey,myval,ierr)
+    ! delete anything in the header that ends in '3'
+    if (len_trim(mykey) > 0) then
+       if (mykey(len_trim(mykey):len_trim(mykey))=='3') then
+          !print*,' deleting ',hdr(i)
+          hdr(i) = ''
+       endif
+    endif
+ enddo
+
+end subroutine flatten_header
+
+!------------------------------------------------
+! get velocity grid from the fits file
+!------------------------------------------------
+subroutine get_velocity_from_fits_header(nv,vel,hdr,ierr)
+ integer, intent(in)  :: nv
+ real(kind=real32), intent(out) :: vel(nv)
+ character(len=80), intent(in)  :: hdr(:)
+ integer, intent(out) :: ierr
+ integer :: k
+ real :: dv,restfreq,crpix,crval,nu
+ character(len=80) :: velocity_type
+ real, parameter :: c = 2.997924e5
+
+ dv = get_from_header('CDELT3',hdr,ierr)
+ restfreq = get_from_header('RESTFRQ',hdr,ierr)
+ velocity_type = get_from_header_s('CTYPE3',hdr,ierr)
+ select case (trim(velocity_type))
+ case("VELO-LSR","VRAD")
+    if (dv > 10.) then
+       dv = dv*1e-3 ! assume m/s
+    endif
+    crpix = get_from_header('CRPIX3',hdr,ierr)
+    crval = get_from_header('CRVAL3',hdr,ierr)
+    do k=1,nv
+       vel(k) = real(crval + dv * (k - crpix),kind=real32)
+    enddo
+ case("FREQ")
+    crpix = get_from_header('CRPIX3',hdr,ierr)
+    crval = get_from_header('CRVAL3',hdr,ierr)
+    do k=1,nv
+       nu = crval + dv*(k - crpix)
+       vel(k) = real(-(nu - restfreq)/restfreq * c,kind=real32)  ! c is in km/s
+    enddo
+ case default
+    write(*,"(4x,a,/)") 'warning: velocity type '//trim(velocity_type)//' not recognised in fits header'
+    dv = 1.
+    do k=1,nv
+       vel(k) = real(dv * (k - 0.5),kind=real32)
+    enddo
+ end select
+
+end subroutine get_velocity_from_fits_header
+
 end module readwrite_fits
